package de.intevation.flys.artifacts.services;

import java.sql.Connection;
import java.sql.SQLException;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import java.util.Map;
import java.util.HashMap;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallMeta;
import de.intevation.artifacts.GlobalContext;
import de.intevation.artifacts.ArtifactDatabase;
import de.intevation.artifacts.ArtifactDatabaseException;

import de.intevation.artifactdatabase.DefaultService;

import de.intevation.artifacts.common.utils.XMLUtils;
import de.intevation.artifacts.common.utils.StringUtils;

import de.intevation.artifacts.common.ArtifactNamespaceContext;

import de.intevation.flys.artifacts.services.meta.Builder;
import de.intevation.flys.artifacts.services.meta.DataCage;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.backend.SessionHolder;

import org.hibernate.Session;

import org.hibernate.jdbc.Work;

public class MetaDataService
extends      DefaultService
{
    private static Logger log = Logger.getLogger(MetaDataService.class);

    // old service. To be removed
    public static final String XPATH_RIVER = "/art:river/text()";

    public static final String XPATH_UUID    = "/art:uuid/@value";
    public static final String XPATH_OUTS    = "/art:outs/@value";
    public static final String XPATH_FILTERS = "/art:filters/@value";

    /** The global context key of the artifact database */
    public static final String ARTIFACT_DATA_BASE_KEY =
        "global.artifact.database";

    public MetaDataService() {
    }

    protected static Map<String, Object> extractParameters(Document data) {
        HashMap<String, Object> parameters = new HashMap<String, Object>();

        String river = XMLUtils.xpathString(
            data, XPATH_RIVER, ArtifactNamespaceContext.INSTANCE);

        if (river == null || (river = river.trim()).length() == 0) {
            river = "%"; // matches all rivers
        }

        parameters.put("river", river);

        return parameters;
    }

    /** The old service. To be removed. */
    protected Document oldService(Document data) {

        final Document result = XMLUtils.newDocument();

        final Builder builder = DataCage.getInstance().getBuilder();

        if (builder == null) {
            log.error("MetaDataService is not setup properly.");
            return result;
        }

        final Map<String, Object> parameters = extractParameters(data);

        Session session = SessionHolder.acquire();
        try {
            session.doWork(new Work() {
                @Override
                public void execute(Connection connection)
                throws SQLException
                {
                    log.debug("MetaDataService.execute");
                    builder.build(connection, result, parameters);
                }
            });
        }
        finally {
            session.close();
            SessionHolder.release();
        }

        return result;
    }

    protected Document newService(
        String        uuid,
        String        outsString,
        String        filters,
        GlobalContext globalContext
    ) {
        Document result = XMLUtils.newDocument();

        if (!StringUtils.checkUUID(uuid)) {
            log.warn("'" + uuid + "' is not a UUID");
            return result;
        }

        Object dbObject =
            (ArtifactDatabase)globalContext.get(ARTIFACT_DATA_BASE_KEY);

        if (!(dbObject instanceof ArtifactDatabase)) {
            log.error("Cannot find artifact database");
            return result;
        }

        ArtifactDatabase db = (ArtifactDatabase)dbObject;

        Artifact artifact;

        try {
            artifact = db.getRawArtifact(uuid);
        }
        catch (ArtifactDatabaseException adbe) {
            log.warn("fetching artifact failed", adbe);
            return result;
        }

        if (!(artifact instanceof FLYSArtifact)) {
            log.warn("artifact is not a FLYS artifact.");
            return result;
        }

        FLYSArtifact flysArtifact = (FLYSArtifact)artifact;

        String [] outs = outsString.split("\\s*,\\s*");

        Map<String, Object> extras;
        
        if (filters != null) {
            extras = new HashMap<String, Object>();
            for (String filter: filters.split("\\s*,\\*s")) {
                extras.put(filter, "true");
            }
        }
        else {
            extras = null;
        }

        DataCage dc = DataCage.getInstance();

        dc.recommend(flysArtifact, outs, extras, result);

        return result;
    }

    @Override
    public Document process(
        Document      data,
        GlobalContext globalContext,
        CallMeta      callMeta
    ) {
        log.debug("MetaDataService.process");

        String uuid = XMLUtils.xpathString(
            data, XPATH_UUID, ArtifactNamespaceContext.INSTANCE);

        String outs = XMLUtils.xpathString(
            data, XPATH_OUTS, ArtifactNamespaceContext.INSTANCE);

        String filters = XMLUtils.xpathString(
            data, XPATH_FILTERS, ArtifactNamespaceContext.INSTANCE);

        return uuid != null && outs != null
            ? newService(uuid, outs, filters, globalContext)
            : oldService(data);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
